#Esaphagus cancer (Minnesota, 87 counties): Analysis using non-spatial random effects:
#Aug 13, 2018

rm(list=ls(all=TRUE))
#---------------
date()
path<-getwd()

runif(1)
seed<-.Random.seed[1]
set.seed(seed, kind ="default")

library(sae)
library(MASS)
library(psych)
library(lme4)
#-----------------------------------------------------------------------------------------------------------------------------------------
#Data (Esaphagus cancer- 87 counties-Minnesota):

#observed:
y=c(14, 52, 8, 16, 9, 0, 20, 12, 12, 8, 10, 1, 8, 11, 1, 0, 3, 
    14, 51, 1, 14, 8, 9, 15, 10, 3, 319, 4, 9, 8, 25, 4, 9, 12, 0, 
    2, 6, 8, 4, 18, 1, 7, 6, 1, 3, 12, 8, 7, 12, 17, 1, 7, 3, 7, 
    33, 25, 6, 11, 7, 12, 3, 161, 1, 13, 8, 16, 4, 3, 98, 10, 3, 
    1, 20, 11, 1, 7, 7, 1, 10, 3, 4, 35, 5, 5, 19, 12, 6)

#Expected:
E=c(7, 49, 11, 11, 9, 3, 16, 11, 11, 13, 11, 6, 10, 15, 4, 2, 6, 
    20, 59, 5, 12, 8, 9, 14, 16, 3, 298, 7, 7, 8, 16, 5, 5, 14, 3, 
    6, 4, 5, 2, 9, 4, 9, 11, 2, 4, 10, 8, 8, 11, 17, 5, 8, 9, 4, 
    29, 23, 5, 9, 5, 13, 5, 147, 2, 8, 8, 15, 4, 5, 77, 14, 10, 6, 
    33, 11, 4, 5, 9, 2, 8, 6, 7, 34, 5, 3, 16, 19, 6)

I<-87
county<-c(1:I)
###################################
#Binomial model
res<-glmer(y ~ 1+(1|county),family="poisson"(link = "log"), offset=log(E)) 

res1<-summary(res)
beta.hat<-c(res1$coefficients[1,1]) #,res1$coefficients[2,1],res1$coefficients[3,1],res1$coefficients[4,1]) 
beta.hat
#[1] -0.06246475
sig2.hat<-(res1$optinfo$val[1])^2
sig2.hat<-ifelse(sig2.hat<=0.0001,0.001,sig2.hat)
sig2.hat
#[1] 0.03255487
#---------------------------------
mspe.te<-rep(0,I)
MSPE<-MU.EB<-MU<-rep(0,I)
mean.dc<-MSE.dc<-sd.dc<-Bias.dc<-RB.dc<-c(rep(0,2))
###############################################
mu.i<-rep(0,1)
Sigma.i<-sig2.hat #*solve(diag(I)-(gamma.hat/max(eigen(Cm)$values))*Cm) 
#Sigma.i <- make.symmetric(cov.i)
#=============================================
#mean.dc<-mean.dc+param
#sd.dc<-sd.dc+sqrt(temp)
#================= 
ETA.temp<-matrix(c(0),1000,1)
for(i in 1:1000){
 ETA.temp[i,]<-rnorm(1,mu.i,sd=sqrt(Sigma.i))
}
ETA<-apply(ETA.temp,2,mean)   

z.tem<-rep(0,I)
for(i in 1:I){z.tem[i]<-(y[i]*exp(-ETA) + ETA -1-beta.hat) 
}

P<-(exp(-ETA))             
mu.EB<-beta.hat+(Sigma.i/(Sigma.i+P))%*%z.tem+log(E)
MU.EB<-MU.EB+mu.EB

print(paste(" ****** mu.EB"))
print(mu.EB)

print(paste(" ****** Relative Risk"))
print(exp(mu.EB)/E)

print(paste(" ****** Naive Relative Risk"))
print(y/E)

print("A")

##########################
sum1=rep(0,2)
sum2=matrix(0,2,2)

for( i in 1:300){
  
 # print(i)
  ETA5.temp<-matrix(c(0),1000,1)
  for(i in 1:1000){
    ETA5.temp[i,]<-rnorm(1,mu.i,sd=sqrt(Sigma.i))
  }
  ETA5<-apply(ETA5.temp,2,mean)
  
  
  mu5<-rep(0,I)
  y.ran5<-rep(0,I)
  for(i in 1:I){
    mu5[i]<-beta.hat+ETA5+log(E[i])
    y.ran5[i]<-rpois(1,exp(mu5[i]))
  }
  #-----------------------------
  res.ran<-glmer(y.ran5 ~ 1+(1|county),family="poisson"(link = "log"), offset=log(E)) 
  
  res1.ran<-summary(res.ran)
  beta.hat.ran<-c(res1.ran$coefficients[1,1]) 
 # beta.hat.ran
  
  sig2.hat.ran<-(res1.ran$optinfo$val[1])^2
  sig2.hat.ran<-ifelse(sig2.hat.ran<=0.0001,0.001,sig2.hat.ran)
#  sig2.hat.ran
  
  sum1=sum1+matrix(c((beta.hat.ran-beta.hat),(sig2.hat.ran-sig2.hat)),2,1)
  sum2=sum2+matrix(c((beta.hat.ran-beta.hat),(sig2.hat.ran-sig2.hat)),2,1)%*%matrix(c((beta.hat.ran-beta.hat),(sig2.hat.ran-sig2.hat)),1,2)
}

Ealpha=sum1/300
Covalpha=sum2/300
###############################    mspe     ###############################################################
##########################################################################################################
gamma.hat<-0 #in the case of non-spatial random effects

Z=diag(I)

R<-Sigma.i+ P
Rdiag<-diag(R)
Sigmasigma2u<-1 
Rsigma2u<-Sigmasigma2u

print("B")

g1sigma2u<-rep(0,I)
g1<-rep(0,I)
g2<-rep(0,I)
g1sigmasigma <-rep(0,I)
Ebeta2<-matrix(0,3*I,3)
Esigma2<-rep(0,I)
g11<-rep(0,I)
g12<-rep(0,I)
mspeta<-rep(0,I)


g1all<-(Sigma.i -(Sigma.i*(1/R)*Sigma.i))
g1<-(g1all)
g1sigma2u<-(Sigmasigma2u -(Sigmasigma2u*(1/R)*Sigma.i) + (Sigma.i*(1/R)*Rsigma2u*(1/R)*Z*Sigma.i) - (Sigma.i*(1/R)*Sigmasigma2u))
g1sigma2udiag=(g1sigma2u)
g1sigmasigma<-((Sigmasigma2u*(1/R)*Rsigma2u*(1/R)*Sigma.i) - (Sigmasigma2u*(1/R)*Sigmasigma2u) + (Sigmasigma2u*(1/R)*Rsigma2u*(1/R)*Sigma.i) - (Sigma.i*(1/R)*Rsigma2u*(1/R)*Rsigma2u*(1/R)*Sigma.i)- (Sigma.i*(1/R)*Rsigma2u*(1/R)*Rsigma2u*(1/R)*Rsigma2u*(1/R)*Sigma.i) + (Sigma.i*(1/R)*Rsigma2u*(1/R)*0) - (Sigmasigma2u*(1/R)*Z*Sigmasigma2u) + (Sigma.i*(1/R)*Rsigma2u*(1/R)*Sigmasigma2u)) 
g1sigmasigmadiag=(g1sigmasigma)

print("C")

sum3=matrix(0,I,I)

for(j in 1:300){
  ETA.temp1<-matrix(c(0),1000,1)
  for(i in 1:1000){
    ETA.temp1[i,]<-rnorm(1,mu.i,sd=sqrt(Sigma.i))
  }
  ETA1<-apply(ETA.temp1,2,mean)
  
  
  mu1<-rep(0,I)
  y.ran1<-rep(0,I)
  z<-rep(0,I)
  for(i in 1:I){
    mu1[i]<-beta.hat+ETA1+log(E[i])  
    y.ran1[i]<-rpois(1,exp(mu1[i]))
    z[i]=(y.ran1[i]*exp(-ETA1) + ETA1 -1-beta.hat) 
  }
  sum3=sum3+matrix(z,I,1)%*%matrix(z,1,I)
 # print(j)
}
Ez=sum3/300

Ebeta2=(1 -  diag(solve(R)%*%Sigma.i))^2
for(i in 1:I){
  Esigma2[i]=(Sigmasigma2u/(Sigma.i + P))*(z[i]^2)*(1/(P+Sigma.i))*(1-2*Sigma.i/(P+Sigma.i))
              }

print("D")

for(i in 1:I){
  k1=matrix(0,1,2)
  k2=matrix(0,1,2)
  k4=matrix(0,2,2)
  
  k2=cbind(0,g1sigmasigmadiag[i])
  k4=rbind(k1,k2)
  g12[i]=tr(k4%*%Covalpha)
  g11[i]=(cbind(0,g1sigmasigmadiag[i]))%*%Ealpha
  g2[i]= tr(diag(c(Ebeta2[i],Esigma2[i]),2,2)%*%Covalpha)
  mspe.te[i]<-g1 - g11[i] - g12[i] + g2[i] 
}
############################################################################################
mspe.te

print(mu.EB)

print(paste(" ****** Relative Risk"))
print(exp(mu.EB)/E)

print(paste(" ****** Naive Relative Risk"))
print(y/E)
##############################################################################################
Result_NonSpatial<-cbind(county,(mu.EB[1,]),exp(mu.EB[1,])/E, mspe.te)
write.table(Result_NonSpatial, "Result_nonspatial.txt")
##############################################################################################