#Simulation study in the case of Poisson mixed model using non-spatial random effects:
#Aug 16, 2018

rm(list=ls(all=TRUE))
#---------------
date()
path<-getwd()

runif(1)
seed<-.Random.seed[1]
set.seed(seed, kind ="default")

library(sae)
library(MASS)
library(psych)
library(lme4)
library(dclone)

#We need adjacency matrix and related spatial information to generate the data:
#adjacency matrix 
adj = c(11,  31,  69,  9,  58,  33,	48,	18,
        27,	71,	30,	13,	82,	62,		
        14,	54,	44,	15,	29,	80,	56,	
        45,	68,	39,	36,	31,	11,	29,	15,	57,
        73,	49,	48,	71,				
        78,	75,	76,	37,				
        22,	46,	83,	8,	52,	40,	81,		
        83,	17,	64,	65,	52,	7,			
        1,	69,	58,						
        27,	70,	72,	43,	86,				
        29,	4,	31,	1,	18,	49,	77,	80,	
        37,	76,	34,	65,	87,				
        82,	2,	30,	58,					
        54,	3,	56,	84,					
        60,	57,	4,	29,	3,	44,			
        38,								
        32,	51,	64,	8,	83,				
        11,	1,	48,	49,					
        25,	66,	70,	27,	62,	82,			
        50,	74,	25,	55,					
        61,	26,	56,	77,					
        46,	7,	81,	24,					
        50,	55,	85,	28,					
        22,	81,	74,	50,					
        79,	55,	20,	66,	19,				
        78,	84,	56,	21,	75,				
        19,	70,	10,	86,	2,	62,			
        23,	85,							
        15,	4,	11,	80,	3,				
        2,	71,	48,	33,	13,				
        4,	36,	69,	1,	11,				
        53,	17,	46,						
        48,	1,	58,	30,					
        12,	76,	61,	73,	47,	65,			
        68,	45,							
        39,	4,	31,	69,					
        6,	12,	87,						
        69,	16,							
        68,     4,      36,						
        81,	7,	52,	72,	70,	66,			
        87,	42,	59,						
        41,	87,	64,	51,					
        10,	72,	65,	47,	86,				
        54,	60,	15,	3,					
        35,	68,	4,	57,	60,				
        32,	83,	7,	22,					
        34,	73,	86,	43,	65,				
        5,	49,	18,	1,	33,	30,	71,		
        77,	11,	18,	48,	5,	73,			
        24,	20,	55,	23,					
        53,	59,	42,	64,	17,				
        7,	8,	65,	72,	40,				
        67,	51,	32,						
        60,	44,	3,	14,					
        79,	85,	23,	50,	20,	25,			
        84,	14,	3,	80,	77,	21,	26,		
        45,	4,	15,	63,	60,				
        13,	33,	1,	9,					
        41,	51,	67,						
        45,	57,	63,	15,	44,	54,			
        76,	75,	21,	73,	34,				
        27,	2,	82,	19,					
        60,	57,							
        17,	51,	42,	87,	65,	8,			
        43,	72,	52,	8,	64,	87,	12,	34,	47,
        74,	81,	40,	70,	19,	25,			
        59,	53,							
        35,	39,	45,	4,					
        36,	31,	1,	9,	38,				
        66,	40,	72,	10,	27,	19,			
        86,	73,	5,	48,	30,	2,			
        65,	43,	10,	70,	40,	52,			
        86,	47,	34,	61,	77,	49,	5,	71,
        24,	81,	66,	20,					
        6,	78,	26,	61,	76,				
        12,	6,	75,	61,	34,				
        21,	56,	80,	11,	49,	73,			
        84,	26,	75,	6,					
        85,	55,	25,						
        56,	3,	29,	11,	77,				
        24,	22,	7,	40,	66,	74,			
        19,	62,	2,	13,					
        46,	17,	8,	7,					
        14,	56,	26,	78,					
        28,	23,	55,	79,					
        47,	73,	71,	27,	10,	43,			
        37,	12,	65,	64,	42,	41)

#number of neighbours of each county:
num = c(8,6,7,9,4,4,7,6,3,5,8,5,4,4,6,1,5,4,6,4,4,4,4,4,5,5,
        6,2,5,5,5,3,4,6,2,4,3,2,3,6,3,4,5,4,5,4,5,7,6,4,5,
        5,3,4,6,7,5,4,3,6,5,4,2,6,9,6,2,4,5,6,6,6,8,4,5,5,6,
        4,3,5,6,4,4,4,4,6,6)
#----------------------------------
#True values:
I<-87
county<-c(1:I)

ss<-rep(0,(I+1))
ss[1]<-1
for(i in 2:(I+1)){ss[i]<-ss[i-1]+num[i-1]}
################Definition of Q and Cm (see Leroux, Lei, Breslow, 1999)###################
Q<-diag(num)
for(i in 1:I){
  a<-ss[i]
  for(k in 1:num[i]){
    Q[i,adj[a+k-1]]<--1
  }
}
Cm<-diag(I)-Q
#=============================================
gamma=0.60
sig2u=0.0001

mu.i<-rep(0,I)
cov.i<-sig2u*solve(diag(I)-(gamma/max(eigen(Cm)$values))*Cm) 
Sigma.i <- make.symmetric(cov.i)
#=============================================
Pop1<-rep(30,I)

beta0=0.001
B=1000

MSPE<-MU.EB<-MU<-mspe.te<-matrix(0,B,I)
BETA.HAT<-SIG2.HAT<-rep(0,B)
######Simulation from  Poisson distribution:#######
for(r in 1:B){
  
  ETA.temp<-matrix(c(0),1000,I)
  for(i in 1:1000){
    ETA.temp[i,]<-mvrnorm(1,mu.i,Sigma.i) 
  }
  ETA<-apply(ETA.temp,2,mean)
  
  
  mu1<-rep(0,I)
  y.ran<-rep(0,I)
  for(i in 1:I){
    mu1[i]<-beta0+ETA[i]+log(Pop1[i])
    y.ran[i]<-rpois(1,exp(mu1[i]))
  }
  
  MU[r,]<-(mu1)
  #-----------------------------
res<-glmer(y.ran ~ 1+(1|county),family="poisson"(link = "log"), offset=log(Pop1)) 

res1<-summary(res)
#print(res1)

beta.hat<-c(res1$coefficients[1,1])  
BETA.HAT[r]<-beta.hat

sig2.hat<-(res1$optinfo$val[1])^2
SIG2.HAT[r]<-sig2.hat
sig2.hat<-ifelse(sig2.hat<=0.000001,0.0001,sig2.hat)
###############################################
mu.i<-rep(0,1)
Sigma.i<-sig2.hat 
#=============================================
ETA.temp0<-matrix(c(0),1000,1)
for(i in 1:1000){
 ETA.temp0[i,]<-rnorm(1,mu.i,sd=sqrt(Sigma.i))
}
ETA0<-apply(ETA.temp0,2,mean)   

z.tem<-rep(0,I)
for(i in 1:I){z.tem[i]<-(y.ran[i]*exp(-ETA0) + ETA0 -1-beta.hat) 
}

P<-(exp(-ETA0))             
mu.EB<-(beta.hat+(Sigma.i/(Sigma.i+P))%*%z.tem+log(Pop1))
MU.EB[r,]<-mu.EB

MSPE[r,]<-(MU.EB[r,]-MU[r,])^2
##########################
sum1=rep(0,2)
sum2=matrix(0,2,2)

for( i in 1:300){
  
 # print(i)
  ETA5.temp<-matrix(c(0),1000,I)
  for(i in 1:1000){
    ETA5.temp[i,]<-rnorm(I,mu.i,sd=sqrt(Sigma.i))
  }
  ETA5<-apply(ETA5.temp,2,mean)
  
  
  mu5<-rep(0,I)
  y.ran5<-rep(0,I)
  for(i in 1:I){
    mu5[i]<-beta.hat+ETA5[i]+log(Pop1[i])
    y.ran5[i]<-rpois(1,exp(mu5[i]))
  }
  #-----------------------------
  res.ran<-glmer(y.ran5 ~ 1+(1|county),family="poisson"(link = "log"), offset=log(Pop1)) 
  
  res1.ran<-summary(res.ran)
  beta.hat.ran<-c(res1.ran$coefficients[1,1]) 

  sig2.hat.ran<-(res1.ran$optinfo$val[1])^2
  sig2.hat.ran<-ifelse(sig2.hat.ran<=0.000001,0.0001,sig2.hat.ran)

  sum1=sum1+matrix(c((beta.hat.ran-beta.hat),(sig2.hat.ran-sig2.hat)),2,1)
  sum2=sum2+matrix(c((beta.hat.ran-beta.hat),(sig2.hat.ran-sig2.hat)),2,1)%*%matrix(c((beta.hat.ran-beta.hat),(sig2.hat.ran-sig2.hat)),1,2)
}

Ealpha=sum1/300
Covalpha=sum2/300
###############################    mspe     ###############################################################
##########################################################################################################
gamma.hat<-0 #in the case of non-spatial random effects

Z=diag(I)

R<-Sigma.i+ P
Rdiag<-diag(R)
Sigmasigma2u<-1 
Rsigma2u<-Sigmasigma2u

#print("B")

g1sigma2u<-rep(0,I)
g1<-rep(0,I)
g2<-rep(0,I)
g1sigmasigma <-rep(0,I)
Ebeta2<-matrix(0,3*I,3)
Esigma2<-rep(0,I)
g11<-rep(0,I)
g12<-rep(0,I)
mspeta<-rep(0,I)


g1all<-(Sigma.i -(Sigma.i*(1/R)*Sigma.i))
g1<-(g1all)
g1sigma2u<-(Sigmasigma2u -(Sigmasigma2u*(1/R)*Sigma.i) + (Sigma.i*(1/R)*Rsigma2u*(1/R)*Z*Sigma.i) - (Sigma.i*(1/R)*Sigmasigma2u))
g1sigma2udiag=(g1sigma2u)
g1sigmasigma<-((Sigmasigma2u*(1/R)*Rsigma2u*(1/R)*Sigma.i) - (Sigmasigma2u*(1/R)*Sigmasigma2u) + (Sigmasigma2u*(1/R)*Rsigma2u*(1/R)*Sigma.i) - (Sigma.i*(1/R)*Rsigma2u*(1/R)*Rsigma2u*(1/R)*Sigma.i)- (Sigma.i*(1/R)*Rsigma2u*(1/R)*Rsigma2u*(1/R)*Rsigma2u*(1/R)*Sigma.i) + (Sigma.i*(1/R)*Rsigma2u*(1/R)*0) - (Sigmasigma2u*(1/R)*Z*Sigmasigma2u) + (Sigma.i*(1/R)*Rsigma2u*(1/R)*Sigmasigma2u)) 
g1sigmasigmadiag=(g1sigmasigma)

#print("C")

sum3=matrix(0,I,I)

for(j in 1:300){
  ETA.temp1<-matrix(c(0),1000,I)
  for(i in 1:1000){
    ETA.temp1[i,]<-rnorm(I,mu.i,sd=sqrt(Sigma.i))
  }
  ETA1<-apply(ETA.temp1,2,mean)
  
  
  mu1<-rep(0,I)
  y.ran1<-rep(0,I)
  z<-rep(0,I)
  for(i in 1:I){
    mu1[i]<-beta.hat+ETA1[i]+log(Pop1[i])  
    y.ran1[i]<-rpois(1,exp(mu1[i]))
    z[i]=(y.ran1[i]*exp(-ETA1[i]) + ETA1[i] -1-beta.hat) 
  }
  sum3=sum3+matrix(z,I,1)%*%matrix(z,1,I)
 # print(j)
}
Ez=sum3/300

Ebeta2=(1 -  diag(solve(R)%*%Sigma.i))^2
for(i in 1:I){
  Esigma2[i]=(Sigmasigma2u/(Sigma.i + P))*(z[i]^2)*(1/(P+Sigma.i))*(1-2*Sigma.i/(P+Sigma.i))
              }

#print("D")

for(i in 1:I){
  k1=matrix(0,1,2)
  k2=matrix(0,1,2)
  k4=matrix(0,2,2)
  
  k2=cbind(0,g1sigmasigmadiag[i])
  k4=rbind(k1,k2)
  g12[i]=tr(k4%*%Covalpha)
  g11[i]=(cbind(0,g1sigmasigmadiag[i]))%*%Ealpha
  g2[i]= tr(diag(c(Ebeta2[i],Esigma2[i]),2,2)%*%Covalpha)
  mspe.te[r,i]<-g1[i] - g11[i] - g12[i] + g2[i] 
}
cat(r)
}# loop of R
############################################################################################
summary(BETA.HAT)
summary(SIG2.HAT)

AMU.EB<-apply(MU.EB,2,mean)
AMSPE<-apply(MSPE,2,mean)
Amspe.te<-apply(mspe.te,2,mean)
##############################################################################################
Result_simulation_Poisson_nonSpatial<-cbind(county,AMU.EB,AMSPE,Amspe.te)
write.table(Result_simulation_Poisson_nonSpatial, "Result_Simulation_Poisson_Lograte_Nonspatial.txt")
##############################################################################################