#Simulation study in the case of Binomial mixed model using Hukum et al. (2018) approach:
#Oct 7, 2018

rm(list=ls(all=TRUE))
#---------------
date()
path<-getwd()

runif(1)
seed<-.Random.seed[1]
set.seed(seed, kind ="default")

library(sae)
library(MASS)
library(psych)
library(lme4)
library(dclone)

#We need adjacency matrix and related spatial information to generate the data:
#adjacency matrix 
adj = c(11,  31,  69,  9,  58,  33,	48,	18,
        27,	71,	30,	13,	82,	62,		
        14,	54,	44,	15,	29,	80,	56,	
        45,	68,	39,	36,	31,	11,	29,	15,	57,
        73,	49,	48,	71,				
        78,	75,	76,	37,				
        22,	46,	83,	8,	52,	40,	81,		
        83,	17,	64,	65,	52,	7,			
        1,	69,	58,						
        27,	70,	72,	43,	86,				
        29,	4,	31,	1,	18,	49,	77,	80,	
        37,	76,	34,	65,	87,				
        82,	2,	30,	58,					
        54,	3,	56,	84,					
        60,	57,	4,	29,	3,	44,			
        38,								
        32,	51,	64,	8,	83,				
        11,	1,	48,	49,					
        25,	66,	70,	27,	62,	82,			
        50,	74,	25,	55,					
        61,	26,	56,	77,					
        46,	7,	81,	24,					
        50,	55,	85,	28,					
        22,	81,	74,	50,					
        79,	55,	20,	66,	19,				
        78,	84,	56,	21,	75,				
        19,	70,	10,	86,	2,	62,			
        23,	85,							
        15,	4,	11,	80,	3,				
        2,	71,	48,	33,	13,				
        4,	36,	69,	1,	11,				
        53,	17,	46,						
        48,	1,	58,	30,					
        12,	76,	61,	73,	47,	65,			
        68,	45,							
        39,	4,	31,	69,					
        6,	12,	87,						
        69,	16,							
        68,     4,      36,						
        81,	7,	52,	72,	70,	66,			
        87,	42,	59,						
        41,	87,	64,	51,					
        10,	72,	65,	47,	86,				
        54,	60,	15,	3,					
        35,	68,	4,	57,	60,				
        32,	83,	7,	22,					
        34,	73,	86,	43,	65,				
        5,	49,	18,	1,	33,	30,	71,		
        77,	11,	18,	48,	5,	73,			
        24,	20,	55,	23,					
        53,	59,	42,	64,	17,				
        7,	8,	65,	72,	40,				
        67,	51,	32,						
        60,	44,	3,	14,					
        79,	85,	23,	50,	20,	25,			
        84,	14,	3,	80,	77,	21,	26,		
        45,	4,	15,	63,	60,				
        13,	33,	1,	9,					
        41,	51,	67,						
        45,	57,	63,	15,	44,	54,			
        76,	75,	21,	73,	34,				
        27,	2,	82,	19,					
        60,	57,							
        17,	51,	42,	87,	65,	8,			
        43,	72,	52,	8,	64,	87,	12,	34,	47,
        74,	81,	40,	70,	19,	25,			
        59,	53,							
        35,	39,	45,	4,					
        36,	31,	1,	9,	38,				
        66,	40,	72,	10,	27,	19,			
        86,	73,	5,	48,	30,	2,			
        65,	43,	10,	70,	40,	52,			
        86,	47,	34,	61,	77,	49,	5,	71,
        24,	81,	66,	20,					
        6,	78,	26,	61,	76,				
        12,	6,	75,	61,	34,				
        21,	56,	80,	11,	49,	73,			
        84,	26,	75,	6,					
        85,	55,	25,						
        56,	3,	29,	11,	77,				
        24,	22,	7,	40,	66,	74,			
        19,	62,	2,	13,					
        46,	17,	8,	7,					
        14,	56,	26,	78,					
        28,	23,	55,	79,					
        47,	73,	71,	27,	10,	43,			
        37,	12,	65,	64,	42,	41)

#number of neighbours of each county:
num = c(8,6,7,9,4,4,7,6,3,5,8,5,4,4,6,1,5,4,6,4,4,4,4,4,5,5,
        6,2,5,5,5,3,4,6,2,4,3,2,3,6,3,4,5,4,5,4,5,7,6,4,5,
        5,3,4,6,7,5,4,3,6,5,4,2,6,9,6,2,4,5,6,6,6,8,4,5,5,6,
        4,3,5,6,4,4,4,4,6,6)
#----------------------------------
#True values:
I<-87
county<-c(1:I)

ss<-rep(0,(I+1))
ss[1]<-1
for(i in 2:(I+1)){ss[i]<-ss[i-1]+num[i-1]}
################Definition of Q and Cm (see Leroux, Lei, Breslow, 1999)###################
Q<-diag(num)
for(i in 1:I){
  a<-ss[i]
  for(k in 1:num[i]){
    Q[i,adj[a+k-1]]<--1
  }
}
Cm<-diag(I)-Q
#=============================================
gamma=0.50
sig2u=0.0001

mu.i<-rep(0,I)
cov.i<-sig2u*solve(diag(I)-(gamma/max(eigen(Cm)$values))*Cm) 
Sigma.i <- make.symmetric(cov.i)
#=============================================
#True values:

beta0=beta1=0.1
B=1000
ni=1 

MSPE<-MU.EB<-MU<-MU.logit<-matrix(0,B,I)
BETA.HAT<-matrix(0,B,2)
SIG2.HAT<-rep(0,B)
x<-runif(I,0,1)
X<-cbind(rep(1,I),x)
#########################################################################
#Functions:glmm_logistic_arealevel

glmm_logistic_arealevel<-function(Region,ys,x,m=m,ni=ni,Ni=Ni){
  
  Region=Region
  Ni=Ni
  ys=ys
  Xs<-cbind(rep(1,m),x) 				# Design matrix of x including first column of 1's
  nd=ni
  
  
  x=Xs 
  p=c(dim(x))[2]
  psi=NULL #0
  psi[1]=0.1
  diff=1
  ottimo=0
  z=diag(1,m)
  beta=matrix(0,p,500)
  beta[,1]=0
  u=matrix(0,m,500)
  u[,1]=0
  betau=rbind(beta,u)   
  xm<-x
  xmz=rbind(t(xm),t(z))
  xmz1=cbind(xm,z)
  Omega=matrix(0,m,m)
  Omega1=matrix(0,m,m)
  
  s=1
  while (diff>0.001){ #0.00001
    s=s+1
    k=1
    Omega1=diag(1,m)
    Omega=psi[s-1]*Omega1
    while (k<=(500-1)){
      k<-k+1
      eta=xm%*%betau[1:p,k-1]+betau[(p+1):(p+m),k-1]
      pd=exp(eta)/(1+exp(eta))
      B=matrix(0,m,m)
      for (i in 1:m) {B[i,i]=nd[i]*pd[i,1]*(1-pd[i,1])}
      zero1=matrix(0,p,p)
      zero2=matrix(0,p,m)
      m1=cbind(zero1,zero2)
      zero3=matrix(0,m,p)
      m2=cbind(zero3,solve(Omega))
      mf=matrix(0,(m+p),(m+p))
      mf=rbind(m1,m2)
      V=xmz%*%B%*%xmz1+mf
      d1=(ys-nd*pd) 
      
      # it is first derivatives of log-likelohood function
      mo=matrix(0,p+m,1)
      zero=matrix(0,p,1)
      ofin=solve(Omega)%*%betau[(p+1):(p+m),k-1]
      mo=rbind(zero,ofin)
      betau[,k]=betau[,k-1]+(solve(V)%*%xmz%*%d1)-solve(V)%*%mo
    }
    
    
    
    betau[,1]=betau[,k]
    bustim=betau[,k]
    etastim=xm%*%bustim[1:p]+bustim[(p+1):(p+m)]
    pd=exp(etastim)/(1+exp(etastim))
    B=matrix(0,m,m)
    for (i in 1:m) {B[i,i]=nd[i]*pd[i,1]*(1-pd[i,1])}
    Ts=solve(solve(Omega)+t(z)%*%B%*%z)
    psi[s]=(1/m)*(sum(diag(Ts%*%solve(Omega1)))+t(bustim[(p+1):(p+m)])%*%solve(Omega1)%*%bustim[(p+1):(p+m)])
    diff=abs(psi[s]-psi[s-1])
  }
  
  
  list(beta.coeff = bustim[1:p],sigma2u=(psi[s]),rand.eff=bustim[(p+1):(p+m)])
}
#---------------------------------------
glmm_logistic_arealevel.EBP.MSE<-function(Region=Region,ys=ys,x=x,start=start,m=m,ni=ni,Ni=Ni,niter=niter){
  mod0=glmm_logistic_arealevel(Region=Region,ys=ys,x=x,start=start,m=m,ni=ni,Ni=Ni,niter=20)
  
  Xs<-cbind(rep(1,m),x) 	
  beta.est <-mod0$beta.coeff
  sigma2u.est<-mod0$sigma2u
  area.eff<-mod0$rand.eff
  eta.est <-Xs%*%beta.est+area.eff
  theta.est<-try(exp(eta.est)/(1+exp(eta.est)),silent = TRUE)
  EBP.Mean<-round(try(theta.est,silent = TRUE),3)
  as=diag(1,m);ar=as  ;z=diag(1,m)
  Hr=matrix(0,m,m)
  for (i in 1:m) {Hr[i,i]=theta.est[i]*(1-theta.est[i])	}		 		
  Zplus=ar%*%Hr%*%z;Xplus=ar%*%Hr%*%Xs
  
  B=matrix(0,m,m)
  for (i in 1:m) {
    B[i,i]=ni[i]*theta.est[i]*(1-theta.est[i])
  }
  
  Ts=solve(solve(mod0$sigma2u*diag(1,m))+t(z)%*%B%*%z)
  T11=solve(t(Xs)%*%B%*%Xs-t(Xs)%*%B%*%z%*%Ts%*%t(z)%*%B%*%Xs)			
  T22=Ts+Ts%*%t(z)%*%B%*%Xs%*%T11%*%t(Xs)%*%t(B)%*%z%*%Ts
  
  G1=diag(Zplus%*%Ts%*%t(Zplus))
  G2=diag((Xplus-Zplus%*%Ts%*%t(z)%*%B%*%Xs)%*%T11%*%t(Xplus-Zplus%*%Ts%*%t(z)%*%B%*%Xs))
  G3<-0
  g3spgrad<-matrix(0,1,m)
  sigmaPlus.Hat=t(z)%*%B%*%z+mod0$sigma2u*t(z)%*%B%*%z%*%t(z)%*%B%*%z              
  g3spgrad1<-(1/mod0$sigma2u^2)*Zplus%*%Ts%*%Ts
  r1=(mod0$sigma2u^(-1))*sum(diag(solve(z)%*%Ts))
  r11=sum(diag(Ts%*%solve(z)%*%Ts%*%solve(z)))
  varpsi=2*((mod0$sigma2u^(-2))*(m-2*r1)+(mod0$sigma2u^(-4))*r11)^(-1)
  
  for (i in 1:m) {
    g3spgrad[1,]<-g3spgrad1[i,]
    G3[i]<-sum(diag(g3spgrad%*%sigmaPlus.Hat%*%t(g3spgrad)*varpsi))
  }
  
  EBP.MSE=round(G1+G2+2*G3,5) 	# Estimate of mse for the proprtions
  CV.EBP= 100*sqrt(EBP.MSE)/EBP.Mean
  
  
  result1=cbind(Region,ni,Ni,ys,EBP.Mean,sqrt(EBP.MSE),CV.EBP,mod0$rand.eff)
  colnames(result1)=c("Region","ni","Ni","ys","EP","EP.SE","EP.CV","Areaeff")
  
  result=list()
  result$Est=result1
  return(result)
}
##################################################

######Simulation from Binomial distribution:#######
for(r in 1:B){
  
  ETA.temp<-matrix(c(0),1000,I)
  for(i in 1:1000){
    ETA.temp[i,]<-mvrnorm(1,mu.i,Sigma.i) 
  }
  ETA<-apply(ETA.temp,2,mean)
  
  
  pi<-rep(0,I)
  Mu<-rep(0,I)
  y.ij<-county<-rep(0,I*ni)
  y.ran<-rep(0,I)
  a<-1
  b<-ni
  for(i in 1:I){
    Mu[i]<-beta0+beta1*x[i]+ETA[i]
    pi[i]<-exp(Mu[i])/(1+exp(Mu[i]))
    y.ij[a:b]<-rbinom(ni,1,pi[i])
    county[a:b]<-rep(i,ni)
    y.ran[i]<-sum(y.ij[a:b])
    a<-b+1
    b<-a+ni-1
  }
  
  MU[r,]<-pi
  MU.logit[r,]<-Mu
  #-----------------------------
  Region<-c(1:I)
  #x<-runif(I,0,1)
  nii<-rep(ni,I)
  #Xs<-cbind(rep(1,I),x)
  
  res<-glmm_logistic_arealevel(Region=Region,ys=y.ran,x=x,m=I,ni=nii,Ni=ni*100)
  beta.est <-res$beta.coeff
  sigma2u.est<-res$sigma2u
  BETA.HAT[r,]<-beta.est
  SIG2.HAT[r]<-sigma2u.est
  
  area.eff<-res$rand.eff
  mu.EB <-X%*%beta.est+area.eff
  
  MU.EB[r,]<-mu.EB
  
  MSPE[r,]<-(mu.EB-Mu)^2
  cat(r)
}#loop of B
##########################
AMSPE<-apply(MSPE,2,mean)
summary(BETA.HAT)
summary(SIG2.HAT)
##############################################################################################
Result_simulation_Binomial_CS<-cbind(county,AMSPE)
write.table(Result_simulation_Binomial_CS, "Result_Simulation_Binomial_CS.txt")
##############################################################################################