#Simulation study in the case of Binomial mixed model using non-spatial random effects:
#Oct 7, 2018

rm(list=ls(all=TRUE))
#---------------
date()
path<-getwd()

runif(1)
seed<-.Random.seed[1]
set.seed(seed, kind ="default")

library(sae)
library(MASS)
library(psych)
library(lme4)
library(Matrix)

#We need adjacency matrix and related spatial information to generate the data:
#adjacency matrix 
adj = c(11,  31,  69,  9,  58,  33,	48,	18,
        27,	71,	30,	13,	82,	62,		
        14,	54,	44,	15,	29,	80,	56,	
        45,	68,	39,	36,	31,	11,	29,	15,	57,
        73,	49,	48,	71,				
        78,	75,	76,	37,				
        22,	46,	83,	8,	52,	40,	81,		
        83,	17,	64,	65,	52,	7,			
        1,	69,	58,						
        27,	70,	72,	43,	86,				
        29,	4,	31,	1,	18,	49,	77,	80,	
        37,	76,	34,	65,	87,				
        82,	2,	30,	58,					
        54,	3,	56,	84,					
        60,	57,	4,	29,	3,	44,			
        38,								
        32,	51,	64,	8,	83,				
        11,	1,	48,	49,					
        25,	66,	70,	27,	62,	82,			
        50,	74,	25,	55,					
        61,	26,	56,	77,					
        46,	7,	81,	24,					
        50,	55,	85,	28,					
        22,	81,	74,	50,					
        79,	55,	20,	66,	19,				
        78,	84,	56,	21,	75,				
        19,	70,	10,	86,	2,	62,			
        23,	85,							
        15,	4,	11,	80,	3,				
        2,	71,	48,	33,	13,				
        4,	36,	69,	1,	11,				
        53,	17,	46,						
        48,	1,	58,	30,					
        12,	76,	61,	73,	47,	65,			
        68,	45,							
        39,	4,	31,	69,					
        6,	12,	87,						
        69,	16,							
        68,     4,      36,						
        81,	7,	52,	72,	70,	66,			
        87,	42,	59,						
        41,	87,	64,	51,					
        10,	72,	65,	47,	86,				
        54,	60,	15,	3,					
        35,	68,	4,	57,	60,				
        32,	83,	7,	22,					
        34,	73,	86,	43,	65,				
        5,	49,	18,	1,	33,	30,	71,		
        77,	11,	18,	48,	5,	73,			
        24,	20,	55,	23,					
        53,	59,	42,	64,	17,				
        7,	8,	65,	72,	40,				
        67,	51,	32,						
        60,	44,	3,	14,					
        79,	85,	23,	50,	20,	25,			
        84,	14,	3,	80,	77,	21,	26,		
        45,	4,	15,	63,	60,				
        13,	33,	1,	9,					
        41,	51,	67,						
        45,	57,	63,	15,	44,	54,			
        76,	75,	21,	73,	34,				
        27,	2,	82,	19,					
        60,	57,							
        17,	51,	42,	87,	65,	8,			
        43,	72,	52,	8,	64,	87,	12,	34,	47,
        74,	81,	40,	70,	19,	25,			
        59,	53,							
        35,	39,	45,	4,					
        36,	31,	1,	9,	38,				
        66,	40,	72,	10,	27,	19,			
        86,	73,	5,	48,	30,	2,			
        65,	43,	10,	70,	40,	52,			
        86,	47,	34,	61,	77,	49,	5,	71,
        24,	81,	66,	20,					
        6,	78,	26,	61,	76,				
        12,	6,	75,	61,	34,				
        21,	56,	80,	11,	49,	73,			
        84,	26,	75,	6,					
        85,	55,	25,						
        56,	3,	29,	11,	77,				
        24,	22,	7,	40,	66,	74,			
        19,	62,	2,	13,					
        46,	17,	8,	7,					
        14,	56,	26,	78,					
        28,	23,	55,	79,					
        47,	73,	71,	27,	10,	43,			
        37,	12,	65,	64,	42,	41)

#number of neighbours of each county:
num = c(8,6,7,9,4,4,7,6,3,5,8,5,4,4,6,1,5,4,6,4,4,4,4,4,5,5,
        6,2,5,5,5,3,4,6,2,4,3,2,3,6,3,4,5,4,5,4,5,7,6,4,5,
        5,3,4,6,7,5,4,3,6,5,4,2,6,9,6,2,4,5,6,6,6,8,4,5,5,6,
        4,3,5,6,4,4,4,4,6,6)
#----------------------------------
#True values:
I<-87
county<-c(1:I)

ss<-rep(0,(I+1))
ss[1]<-1
for(i in 2:(I+1)){ss[i]<-ss[i-1]+num[i-1]}
################Definition of Q and Cm (see Leroux, Lei, Breslow, 1999)###################
Q<-diag(num)
for(i in 1:I){
  a<-ss[i]
  for(k in 1:num[i]){
    Q[i,adj[a+k-1]]<--1
  }
}
Cm<-diag(I)-Q
#=============================================
gamma=0.50
sig2u=0.0001

mu.i<-rep(0,I)
cov.i<-sig2u*solve(diag(I)-(gamma/max(eigen(Cm)$values))*Cm) 
#Sigma.i <- make.symmetric(cov.i)
Sigma.i <-forceSymmetric(cov.i)
#=============================================
B=1000
ni=1
beta0=beta1=0.1

MSPE<-MSPE.logit<-PI.EB<-PI<-mspe.te<-matrix(0,B,I)
BETA.HAT<-matrix(0,B,2)
SIG2.HAT<-rep(0,B)
MC=500 #number of monte carlo for approximation of expectation needed for the mspe (Taylor)
x<-runif(I,0,1)
X<-cbind(rep(1,I),x)
######Simulation from  Poisson distribution:#######
for(r in 1:B){
  
  ETA.temp<-matrix(c(0),1000,I)
  for(i in 1:1000){
    ETA.temp[i,]<-mvrnorm(1,mu.i,Sigma.i) 
  }
  ETA<-apply(ETA.temp,2,mean)
  
  MU<-rep(0,I)
  pi<-rep(0,I)
  y.ij<-county<-rep(0,I*ni)
  y.ran<-rep(0,I)
  a<-1
  b<-ni
  for(i in 1:I){
    MU[i]<-beta0+beta1*x[i]+ETA[i]
    pi[i]<-exp(MU[i])/(1+exp(MU[i]))
    y.ij[a:b]<-rbinom(ni,1,pi[i])
    county[a:b]<-rep(i,ni)
    y.ran[i]<-sum(y.ij[a:b])
    a<-b+1
    b<-a+ni-1
   }
  
  PI[r,]<-pi
 #-----------------------------
res<-glmer(y.ij ~ x+(1|county),family="binomial"(link = "logit")) 

res1<-summary(res)
beta.hat<-c(res1$coefficients[1,1],res1$coefficients[2,1])  
BETA.HAT[r,]<-beta.hat

sig2.hat<-(res1$optinfo$val[1])^2
SIG2.HAT[r]<-sig2.hat
sig2.hat<-ifelse(sig2.hat<=0.000001,0.0001,sig2.hat)
###############################################
muu.i<-rep(0,1)
Sigma.i<-sig2.hat*diag(I) 
#=============================================
ETA.temp0<-matrix(c(0),1000,I)
for(i in 1:1000){
 ETA.temp0[i,]<-rnorm(I,muu.i,sd=sqrt(sig2.hat))
}
ETA0<-apply(ETA.temp0,2,mean)   

z.tem<-rep(0,I)
for(i in 1:I){z.tem[i]<-(y.ran[i]*exp(-ETA0[i])*(1+exp(ETA0[i]))^2 -(1+exp(ETA0[i]))+ ETA0[i] -X[i,]%*%beta.hat) 
}

P<-diag(exp(-ETA0)*(1+exp(ETA0))^2) 
mu.EB<-X%*%beta.hat+Sigma.i%*%(solve(Sigma.i+P))%*%z.tem
PI.EB[r,]<-exp(mu.EB)/(1+exp(mu.EB))

MSPE[r,]<-(PI.EB[r,]-PI[r,])^2
MSPE.logit[r,]<-(mu.EB-MU)^2

cat(r)
}#loop of R
############################################################################################
summary(BETA.HAT)
summary(SIG2.HAT)

API.EB<-apply(PI.EB,2,mean)
AMSPE<-apply(MSPE,2,mean)
AMSPE.logit<-apply(MSPE.logit,2,mean)
##############################################################################################
county1<-c(1:I)
Result_simulation_Binomial_nonSpatial<-cbind(county1,API.EB,AMSPE,AMSPE.logit)
write.table(Result_simulation_Binomial_nonSpatial, "Result_Simulation_Binomial_Logit_Nonspatial.txt")
##############################################################################################




